﻿using System;
using System.IO;
using System.Threading;
using Ozeki.VoIP;

namespace _09_Predictive_Dialer
{
    class Program
    {
        static Softphone _mySoftphone;
        static string _path;
        static CallInfo _callInfo;
        static Autodialer _autodialer;
        static int _maxConcurrentCall;
        static int _memberCount;
        static ConfigStore _configStore;
        
        static void Main(string[] args)
        {
            InitSoftphone();
            ShowGreetingMessage();
            ReadRegisterInfos();

            BlockExit();
        }

        static void InitSoftphone()
        {
            _configStore = ConfigStore.Instance;
            _mySoftphone = new Softphone();
            _mySoftphone.PhoneLineStateChanged += mySoftphone_PhoneLineStateChanged;
            _path = @"../../Resources/ExampleCSV.csv";
        }


        static void mySoftphone_PhoneLineStateChanged(object sender, RegistrationStateChangedArgs e)
        {
            Console.WriteLine("Phone line state changed: {0}", e.State);

            if (e.State == RegState.Error || e.State == RegState.NotRegistered)
                ReadRegisterInfos();

            if (e.State == RegState.RegistrationSucceeded)
            {
                Console.WriteLine("Registration succeeded - Online!\n");
                AgentAdder();
                GetMaxConcurrentCall();
                GetCSV();
            }
        }

        static void AgentAdder()
        {
            _memberCount = 0;

            while (_memberCount == 0)
            {
                Console.Write("\nThe number of the RingGroup members: ");
                try
                {
                    _memberCount = int.Parse(Console.ReadLine());
                }
                catch
                {
                    Console.WriteLine("Wrong input!");
                }
            }

            for (int i = 0; i < _memberCount; i++)
            {
                Console.Write("{0}. member's phone number: ", i + 1);
                string member = Console.ReadLine();
                _configStore.AddAgent(member);
                _configStore.AddFreeAgent(member);
            }
        }

        static void GetMaxConcurrentCall()
        {
            while (_maxConcurrentCall == 0)
            {
                try
                {
                    Console.Write("\nPlease enter how many calls can be made simultaneously: ");
                    _maxConcurrentCall = int.Parse(Console.ReadLine());
                }
                catch
                {
                    Console.WriteLine("Wrong input. Please enter an integer number.");
                }
            }
            if (_maxConcurrentCall > _memberCount)
            {
                Console.WriteLine("The maximum of concurrent calls can't be larger than the agent's number.");
                _maxConcurrentCall = _memberCount;
                Console.WriteLine("The maximum of concurrent calls is: {0}.", _maxConcurrentCall);
            }
        }

        static void GetCSV()
        {
            Console.Write("\nPlease enter a path to a .csv file (f.e.: C:\\ExampleCSV.csv) or press Enter to load deafult file: ");
            string getPath = Console.ReadLine();

            if (!string.IsNullOrEmpty(getPath))
            {
                _path = getPath;
            }

            var fileI = new FileInfo(_path);
            Console.WriteLine("\nThe file's path is: {0}.\n", fileI.FullName);

            ReadCSV();
        }

        static void ReadCSV()
        {
            try
            {
                using (StreamReader sr = new StreamReader(_path))
                {
                    string line;
                    while ((line = sr.ReadLine()) != null)
                    {
                        ParseCSVLineToObjectList(line);
                    }
                    ShowList();
                }
                StartAutodialer();
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error occured: {0}", ex.Message.ToString());
            }
        }

        static void ParseCSVLineToObjectList(string line)
        {
            try
            {
                string[] parse = line.Split(',', ';');
                _callInfo = new CallInfo(parse[0], parse[1]);
                _configStore.AddToCallList(_callInfo);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error occured: {0}", ex.Message);
            }
        }

        static void ShowList()
        {
            Console.WriteLine();
            foreach (var callListMember in _configStore.GetCallList())
            {
                Console.WriteLine("Phone number: \"{0}\", message: \"{1}\".", callListMember.PhoneNumber, callListMember.Message);
            }
            Console.WriteLine();
        }

        static void StartAutodialer()
        {
            _autodialer = new Autodialer(_mySoftphone, _configStore.GetCallList(), _maxConcurrentCall);
            _autodialer.Start();
        }


        static void ShowGreetingMessage()
        {
            Console.WriteLine("This is an Ozeki VoIP SIP SDK demo written in C#.");
            Console.WriteLine("This example will introduce the followings:");
            Console.WriteLine("- asks the user about sip account information");
            Console.WriteLine("- asks the user about the ring group member agents");
            Console.WriteLine("- asks the user about the amount of calls can be made simultaneously");
            Console.WriteLine("- asks the user about a .csv file path (default option is also provided), parses the file " +
                              "to separate phone numbers from messages");
            Console.WriteLine("- calls and connects the clients and the the ring group members.");
            Console.WriteLine("-------------------------------------------------------------------------------");
            Console.WriteLine();
        }


        static void ReadRegisterInfos()
        {
            var registrationRequired = true;
            Console.WriteLine("\nPlease set up Your SIP account:\n");

            // Asks, if a registration is required to the PBX. The default value is true.
            Console.Write("Please set if the registration is required (true/false) (default: true): ");
            var regRequired = Read("Registration required", false);
            if (regRequired.ToLower() == "false" || regRequired.ToLower() == "no" ||
                 regRequired.ToLower() == "n")
            {
                registrationRequired = false;
            }
            else
            {
                Console.WriteLine("Registration set to required.");
            }


            // The SIP account needs and authentication ID, and some names as well.
            Console.Write("Please set Your authentication ID: ");
            var authenticationId = Read("Authentication ID", true);

            // If the user only presses the Enter button, the username will be the same as the authentication ID
            Console.Write("Please set Your username (default: " + authenticationId + "): ");
            var userName = Read("Username", false);
            if (string.IsNullOrEmpty(userName))
                userName = authenticationId;

            // If the user only presses the Enter button, the display name will be the same as the authentication ID
            Console.Write("Please set Your name to be displayed (default: " + authenticationId + "): ");
            var displayName = Read("Display name", false);
            if (string.IsNullOrEmpty(displayName))
                displayName = authenticationId;

            // The registration password needs to be entered.
            Console.Write("Please set Your registration password: ");
            var registerPassword = Read("Password", true);

            // Domain name as a string, for example an IP adress.
            Console.Write("Please set the domain name: ");
            var domainHost = Read("Domain name", true);

            // Port number with the as 5060 default value.
            Console.Write("Please set the port number (default: 5060): ");
            int domainPort;
            string port = Read("Port", false);
            if (string.IsNullOrEmpty(port))
            {
                domainPort = 5060;
            }
            else
            {
                domainPort = int.Parse(port);
            }
            Console.WriteLine("\nCreating SIP account and trying to register....\n");

            _mySoftphone.Register(registrationRequired, displayName, userName, authenticationId, registerPassword, domainHost, domainPort);
        }

        static string Read(string inputName, bool readWhileEmpty)
        {
            while (true)
            {
                string input = Console.ReadLine();

                if (!readWhileEmpty)
                    return input;

                if (!string.IsNullOrEmpty(input))
                    return input;

                Console.WriteLine(inputName + " cannot be empty.");
                Console.Write(inputName + ": ");
            }
        }


        static void BlockExit()
        {
            while (true)
            {
                Thread.Sleep(10);
            }
        }
    }
}
